@@ -561,6 +561,8 @@ load_image(int no, const char *name)
     char           *ptr;
     Drawable        draw;
 
+    prog_x11_set_title(win, name);
+
     Vprintf("Show  %d: '%s'\n", no, name);
 
     anim_update(NULL, NULL, NULL, NULL, 0);     /* Clean up previous animation */
@@ -694,15 +696,15 @@ main(int argc, char **argv)
             break;
         case 's':              /* Scale output (window size wrt. image size) */
             opt_scale = true;
-            opt_sc_out_y = 0.f;
+            opt_sc_out_y = 0.;
             sscanf(optarg, "%lf,%lf", &opt_sc_out_x, &opt_sc_out_y);
-            if (opt_sc_out_y == 0.f)
+            if (opt_sc_out_y == 0.)
                 opt_sc_out_y = opt_sc_out_x;
             break;
         case 'S':              /* Scale input (input imgage, grab) */
             opt_sc_inp_y = 0.f;
             sscanf(optarg, "%lf,%lf", &opt_sc_inp_x, &opt_sc_inp_y);
-            if (opt_sc_inp_y == 0.f)
+            if (opt_sc_inp_y == 0.)
                 opt_sc_inp_y = opt_sc_inp_x;
             break;
         case 't':
@@ -804,7 +806,7 @@ main(int argc, char **argv)
             case KeyPress:
                 while (XCheckTypedWindowEvent(disp, win, KeyPress, &ev))
                     ;
-                key = XLookupKeysym(&ev.xkey, 0);
+                key = XLookupKeysym(&ev.xkey, ev.xkey.state & ShiftMask);
                 switch (key)
                 {
                 default:
@@ -817,6 +819,10 @@ main(int argc, char **argv)
                     goto quit;
                 case XK_r:
                     goto show_cur;
+                case XK_R:
+                    opt_scale = true;
+                    opt_sc_out_x = opt_sc_out_y = 1.;
+                    goto show_cur;
                 case XK_Right:
                     goto show_next;
                 case XK_Left:
@@ -921,6 +927,16 @@ main(int argc, char **argv)
                     image_width = 0;    /* Reset window size */
                 load_image_frame(argv[no], finfo.frame_num, inc);
                 break;
+            case ConfigureNotify:
+                while (XCheckTypedWindowEvent(disp, win, ConfigureNotify, &ev))
+                    ;
+                if (ev.xconfigure.width == window_width &&
+                    ev.xconfigure.height == window_height)
+                    break;
+                opt_scale = true;
+                opt_sc_out_x = (double)ev.xconfigure.width / image_width;
+                opt_sc_out_y = (double)ev.xconfigure.height / image_height;
+                goto show_cur;
             }
         }
         while (XPending(disp));
